% gene_clustering_EUCdata()
% 
%  Clustering gene trees generatd by simulated sequences
%    
%   This matlab code is used for generaitng the clustering accuracy and
%     and its graphs for the article:
%      "Multi-locus Phylogenetic Analysis with Gene Tree Clustering"
%      by Ruriko Yoshida, Kenji Fukumizu, and Chrysafis Vogiatzis
%
%    INPUT: the data files are given in the form of distance matrices
%    OUTPUT: clustering accuracy and graphs

%   Required toolboxes by others
%     drtoolbox:  by van der Maaten
%             https://lvdmaaten.github.io/drtoolbox/
%     Ncut_9:  by Timothee Cour, Stella Yu, Jianbo Shi
%             http://www.timotheecour.com/software/ncut/ncut.html
%
%   Date: Feb 24, 2016
%   Version: 1.0
%   Author: Kenji Fukumizu
%   Affiliation:  The Institute of Statistical Mathematics, ROIS
%   (c) Kenji Fukumizu
%-------------------------------------------------------
%
%   function gene_clustering_simdata(method,ncluter)
%
%   method: clustering method "ncut", "kmean", "hclust"
%   nclust: the number of clusters
%

function gene_clustering_EUCdata(method,nclust)

SAVEIMG=1;
SAVE_DIR='ResultsImg';


% clustetring batch 

method_list={'Ncut','Kmean','Hclust'};

close all;

if ~ismember(method,method_list)
    error('method must be Ncut, Kmean, Hclust');
end
if nclust~=2
    error('current version allows only two clusters');
end

depth=[0.6 0.7 0.8 0.9 1 1.2 1.4 1.6 1.8 2];    % ratio for depth
%dr={'kpca','tsne','isomap','direct'};   
dr={'kpca','tsne','direct'};   
type={'NJp','GTR', 'HKY', 'K80', 'JC'};   % four methods for constuctiong gene trees. 
                %  NJp is neighborhood joining and the others are maximum likelihood

dim=3;  % dimension used by dimension reduction methods  (usually 2 or 3)



cls=cell(length(depth),length(type),length(dr));
for i=1:length(depth)
    for j=1:length(type)
        for k=1:length(dr)  
            clst_result=clustering_genetree_EUC(depth(i),method,type{j},dr{k},dim,nclust);            
            cls{i,j,k}=clst_result;
        end        
    end
end



% Evaluation and display graph
cmap=lines(5);
cmap=cmap([3 1 2 4 5],:);
lstyle={'-',':','-.','--'};
figure('Position',[100,500,1000,500]);

hold on;
for j=1:length(type)
    for k=1:length(dr)         
        rate=zeros(length(depth),1);
        fprintf('\nEuc: %s %s, D=%d + %s\n\n', type{j},dr{k},dim,method);
        for i=1:length(depth)
            cls_tbl=cls{i,j,k};
            [n d]=size(cls_tbl);
            n1=n/2;
            truecls=ones(n1,1);
            truecls=[truecls; zeros(n1,1)];
            numerror=nnz(cls_tbl(:,1)-truecls);
            correct=(n-numerror)/n;
            if correct<0.5          % valid only for two clusters
                correct=1-correct;
            end
            rate(i)=correct;
            fprintf('%.1f\t%.3f\n',depth(i),rate(i));
        end
        if strcmp(type(j),'NJp')
            plot(depth,rate,'Marker','o',...
                'LineStyle',lstyle{k},'Color',cmap(1,:),'LineWidth',2);
        else
            plot(depth,rate,'Marker','o',...
                'LineStyle',lstyle{k},'Color',cmap(j,:),'LineWidth',2);
        end
    end
end

set(gca, 'FontName','Times','FontSize',14);
ttl=sprintf('Clustering accuracy, %s, Euclidean (NJp, MLE-GTR/HKY/K80/JC)',method);
title(ttl,'FontName','Times','FontSize',14);
if strcmp(method,'Kmean')
    legh=legend('NJp KPCA','NJp, t-SNE', 'GTR, KPCA','GTR, t-SNE','HKY, KPCA','HKY, t-SNE',  'K80, KPCA','K80, t-SNE', 'JC, KPCA','JC, t-SNE', 'Location','NorthEastOutside' );
else
    legh=legend('NJp KPCA','NJp, t-SNE','NJp, direct', 'GTR, KPCA','GTR, t-SNE','GTR, direct','HKY, KPCA','HKY, t-SNE', 'HKY, direct', 'K80, KPCA','K80, t-SNE', 'K80, direct','JC, KPCA','JC, t-SNE', 'JC, direct','Location','NorthEastOutside' );
end
xlabel('c (ratio in depth)');
ylabel('Accuracy')
axis([0.6 2 0.5 1]);
%set(legh,'Location','NorthOutside','FontName','Times','FontSize',14);


if SAVEIMG
    imgfile=sprintf('%s/%s_Euc_results_graph2.eps',SAVE_DIR,method);
    saveas(gcf, imgfile,'psc2');
    imgfile=sprintf('%s/%s_Euc_results_graph2.fig',SAVE_DIR,method);
    saveas(gcf, imgfile,'fig');
end


end


